@@ -12,21 +12,14 @@ module Grape
 
     class << self
       def new(*args, &block)
-        if self == Endpoint
-          Class.new(Endpoint).new(*args, &block)
-        else
-          super
-        end
+        self == Endpoint ? Class.new(Endpoint).new(*args, &block) : super
       end
 
       def before_each(new_setup = false, &block)
         @before_each ||= []
         if new_setup == false
-          if block_given?
-            @before_each << block
-          else
-            return @before_each
-          end
+          return @before_each unless block_given?
+          @before_each << block
         else
           @before_each = [new_setup]
         end
@@ -34,9 +27,7 @@ module Grape
 
       def run_before_each(endpoint)
         superclass.run_before_each(endpoint) unless self == Endpoint
-        before_each.each do |blk|
-          blk.call(endpoint) if blk.respond_to? :call
-        end
+        before_each.each { |blk| blk.call(endpoint) if blk.respond_to?(:call) }
       end
 
       # @api private
@@ -53,8 +44,8 @@ module Grape
       # @return [Proc]
       # @raise [NameError] an instance method with the same name already exists
       def generate_api_method(method_name, &block)
-        if instance_methods.include?(method_name.to_sym) || instance_methods.include?(method_name.to_s)
-          fail NameError.new("method #{method_name.inspect} already exists and cannot be used as an unbound method name")
+        if method_defined?(method_name)
+          raise NameError.new("method #{method_name.inspect} already exists and cannot be used as an unbound method name")
         end
 
         define_method(method_name, &block)
@@ -69,6 +60,18 @@ module Grape
       end
     end
 
+    # Create a new endpoint.
+    # @param new_settings [InheritableSetting] settings to determine the params,
+    #   validations, and other properties from.
+    # @param options [Hash] attributes of this endpoint
+    # @option options path [String or Array] the path to this endpoint, within
+    #   the current scope.
+    # @option options method [String or Array] which HTTP method(s) can be used
+    #   to reach this endpoint.
+    # @option options route_options [Hash]
+    # @note This happens at the time of API definition, so in this context the
+    # endpoint does not know if it will be mounted under a different endpoint.
+    # @yield a block defining what your API should do when this endpoint is hit
     def initialize(new_settings, options = {}, &block)
       require_option(options, :path)
       require_option(options, :method)
@@ -90,6 +93,13 @@ module Grape
       @options[:route_options] ||= {}
 
       @lazy_initialize_lock = Mutex.new
+      @lazy_initialized = nil
+      @block = nil
+
+      @status = nil
+      @file = nil
+      @body = nil
+      @proc = nil
 
       return unless block_given?
 
@@ -97,16 +107,30 @@ module Grape
       @block = self.class.generate_api_method(method_name, &block)
     end
 
+    # Update our settings from a given set of stackable parameters. Used when
+    # the endpoint's API is mounted under another one.
+    def inherit_settings(namespace_stackable)
+      inheritable_setting.route[:saved_validations] += namespace_stackable[:validations]
+      parent_declared_params = namespace_stackable[:declared_params]
+
+      if parent_declared_params
+        inheritable_setting.route[:declared_params] ||= []
+        inheritable_setting.route[:declared_params].concat(parent_declared_params.flatten)
+      end
+
+      endpoints && endpoints.each { |e| e.inherit_settings(namespace_stackable) }
+    end
+
     def require_option(options, key)
-      fail Grape::Exceptions::MissingOption.new(key) unless options.key?(key)
+      raise Grape::Exceptions::MissingOption.new(key) unless options.key?(key)
     end
 
     def method_name
       [options[:method],
        Namespace.joined_space(namespace_stackable(:namespace)),
        (namespace_stackable(:mount_path) || []).join('/'),
-       options[:path].join('/')
-      ].join(' ')
+       options[:path].join('/')]
+        .join(' ')
     end
 
     def routes
@@ -171,12 +195,12 @@ module Grape
 
     def prepare_version
       version = namespace_inheritable(:version) || []
-      return if version.length == 0
+      return if version.empty?
       version.length == 1 ? version.first.to_s : version
     end
 
-    def merge_route_options(default = {})
-      options[:route_options].clone.reverse_merge(default)
+    def merge_route_options(**default)
+      options[:route_options].clone.reverse_merge(**default)
     end
 
     def map_routes
@@ -218,80 +242,81 @@ module Grape
     def run
       ActiveSupport::Notifications.instrument('endpoint_run.grape', endpoint: self, env: env) do
         @header = {}
-
-        @request = Grape::Request.new(env)
+        @request = Grape::Request.new(env, build_params_with: namespace_inheritable(:build_params_with))
         @params = @request.params
         @headers = @request.headers
 
         cookies.read(@request)
-
         self.class.run_before_each(self)
-
         run_filters befores, :before
 
-        run_filters before_validations, :before_validation
+        if (allowed_methods = env[Grape::Env::GRAPE_ALLOWED_METHODS])
+          raise Grape::Exceptions::MethodNotAllowed, header.merge('Allow' => allowed_methods) unless options?
+          header 'Allow', allowed_methods
+          response_object = ''
+          status 204
+        else
+          run_filters before_validations, :before_validation
+          run_validators validations, request
+          run_filters after_validations, :after_validation
+          response_object = @block ? @block.call(self) : nil
+        end
 
-        run_validators validations, request unless env[Grape::Env::GRAPE_METHOD_NOT_ALLOWED]
-
-        run_filters after_validations, :after_validation
-
-        response_object = @block ? @block.call(self) : nil
         run_filters afters, :after
         cookies.write(header)
 
-        # The Body commonly is an Array of Strings, the application instance itself, or a File-like object.
-        response_object = file || [body || response_object]
+        # status verifies body presence when DELETE
+        @body ||= response_object
+
+        # The Body commonly is an Array of Strings, the application instance itself, or a File-like object
+        response_object = file || [body]
         [status, header, response_object]
       end
     end
 
-    def build_stack
-      b = Rack::Builder.new
+    def build_stack(helpers)
+      stack = Grape::Middleware::Stack.new
 
-      b.use Rack::Head
-      b.use Grape::Middleware::Error,
-            format: namespace_inheritable(:format),
-            content_types: namespace_stackable_with_hash(:content_types),
-            default_status: namespace_inheritable(:default_error_status),
-            rescue_all: namespace_inheritable(:rescue_all),
-            default_error_formatter: namespace_inheritable(:default_error_formatter),
-            error_formatters: namespace_stackable_with_hash(:error_formatters),
-            rescue_options: namespace_stackable_with_hash(:rescue_options) || {},
-            rescue_handlers: namespace_stackable_with_hash(:rescue_handlers) || {},
-            base_only_rescue_handlers: namespace_stackable_with_hash(:base_only_rescue_handlers) || {},
-            all_rescue_handler: namespace_inheritable(:all_rescue_handler)
+      stack.use Rack::Head
+      stack.use Class.new(Grape::Middleware::Error),
+                helpers: helpers,
+                format: namespace_inheritable(:format),
+                content_types: namespace_stackable_with_hash(:content_types),
+                default_status: namespace_inheritable(:default_error_status),
+                rescue_all: namespace_inheritable(:rescue_all),
+                rescue_grape_exceptions: namespace_inheritable(:rescue_grape_exceptions),
+                default_error_formatter: namespace_inheritable(:default_error_formatter),
+                error_formatters: namespace_stackable_with_hash(:error_formatters),
+                rescue_options: namespace_stackable_with_hash(:rescue_options) || {},
+                rescue_handlers: namespace_reverse_stackable_with_hash(:rescue_handlers) || {},
+                base_only_rescue_handlers: namespace_stackable_with_hash(:base_only_rescue_handlers) || {},
+                all_rescue_handler: namespace_inheritable(:all_rescue_handler)
 
-      (namespace_stackable(:middleware) || []).each do |m|
-        m = m.dup
-        block = m.pop if m.last.is_a?(Proc)
-        block ? b.use(*m, &block) : b.use(*m)
-      end
+      stack.concat namespace_stackable(:middleware)
 
       if namespace_inheritable(:version)
-        b.use Grape::Middleware::Versioner.using(namespace_inheritable(:version_options)[:using]),
-              versions: namespace_inheritable(:version) ? namespace_inheritable(:version).flatten : nil,
-              version_options: namespace_inheritable(:version_options),
-              prefix: namespace_inheritable(:root_prefix)
-
+        stack.use Grape::Middleware::Versioner.using(namespace_inheritable(:version_options)[:using]),
+                  versions: namespace_inheritable(:version) ? namespace_inheritable(:version).flatten : nil,
+                  version_options: namespace_inheritable(:version_options),
+                  prefix: namespace_inheritable(:root_prefix),
+                  mount_path: namespace_stackable(:mount_path).first
       end
 
-      b.use Grape::Middleware::Formatter,
-            format: namespace_inheritable(:format),
-            default_format: namespace_inheritable(:default_format) || :txt,
-            content_types: namespace_stackable_with_hash(:content_types),
-            formatters: namespace_stackable_with_hash(:formatters),
-            parsers: namespace_stackable_with_hash(:parsers)
+      stack.use Grape::Middleware::Formatter,
+                format: namespace_inheritable(:format),
+                default_format: namespace_inheritable(:default_format) || :txt,
+                content_types: namespace_stackable_with_hash(:content_types),
+                formatters: namespace_stackable_with_hash(:formatters),
+                parsers: namespace_stackable_with_hash(:parsers)
 
-      b.run ->(env) { env[Grape::Env::API_ENDPOINT].run }
-
-      b.to_app
+      builder = stack.build
+      builder.run ->(env) { env[Grape::Env::API_ENDPOINT].run }
+      builder.to_app
     end
 
     def build_helpers
       helpers = namespace_stackable(:helpers) || []
-      Module.new do
-        helpers.each { |mod_to_include| include mod_to_include }
-      end
+      Module.new { helpers.each { |mod_to_include| include mod_to_include } }
     end
 
     private :build_stack, :build_helpers
@@ -306,27 +331,33 @@ module Grape
       @lazy_initialize_lock.synchronize do
         return true if @lazy_initialized
 
-        @app = options[:app] || build_stack
-        @helpers = build_helpers.tap do |mod|
-          self.class.send(:include, mod)
-        end
+        @helpers = build_helpers.tap { |mod| self.class.send(:include, mod) }
+        @app = options[:app] || build_stack(@helpers)
 
         @lazy_initialized = true
       end
     end
 
-    def run_validators(validators, request)
+    def run_validators(validator_factories, request)
       validation_errors = []
 
-      validators.each do |validator|
-        begin
-          validator.validate(request)
-        rescue Grape::Exceptions::Validation => e
-          validation_errors << e
+      validators = validator_factories.map(&:create_validator)
+
+      ActiveSupport::Notifications.instrument('endpoint_run_validators.grape', endpoint: self, validators: validators, request: request) do
+        validators.each do |validator|
+          begin
+            validator.validate(request)
+          rescue Grape::Exceptions::Validation => e
+            validation_errors << e
+            break if validator.fail_fast?
+          rescue Grape::Exceptions::ValidationArrayErrors => e
+            validation_errors += e.errors
+            break if validator.fail_fast?
+          end
         end
       end
 
-      validation_errors.any? && fail(Grape::Exceptions::ValidationErrors, errors: validation_errors, headers: header)
+      validation_errors.any? && raise(Grape::Exceptions::ValidationErrors, errors: validation_errors, headers: header)
     end
 
     def run_filters(filters, type = :other)
@@ -356,5 +387,10 @@ module Grape
     def validations
       route_setting(:saved_validations) || []
     end
+
+    def options?
+      options[:options_route_enabled] &&
+        env[Grape::Http::Headers::REQUEST_METHOD] == Grape::Http::Headers::OPTIONS
+    end
   end
 end
