@@ -32,7 +32,7 @@ module Grape
         def before
           strict_header_checks if strict?
 
-          if media_type || env[Grape::Env::GRAPE_METHOD_NOT_ALLOWED]
+          if media_type || env[Grape::Env::GRAPE_ALLOWED_METHODS]
             media_type_header_handler
           elsif headers_contain_wrong_vendor?
             fail_with_invalid_accept_header!('API vendor not found.')
@@ -87,12 +87,12 @@ module Grape
         end
 
         def fail_with_invalid_accept_header!(message)
-          fail Grape::Exceptions::InvalidAcceptHeader
+          raise Grape::Exceptions::InvalidAcceptHeader
             .new(message, error_headers)
         end
 
         def fail_with_invalid_version_header!(message)
-          fail Grape::Exceptions::InvalidVersionHeader
+          raise Grape::Exceptions::InvalidVersionHeader
             .new(message, error_headers)
         end
 
@@ -159,7 +159,7 @@ module Grape
         # header, one can set the `:cascade` option to `false`.
         def cascade?
           if version_options && version_options.key?(:cascade)
-            !!version_options[:cascade]
+            version_options[:cascade]
           else
             true
           end
