@@ -25,26 +25,34 @@ module Grape
 
         def before
           path = env[Grape::Http::Headers::PATH_INFO].dup
+          path.sub!(mount_path, '') if mounted_path?(path)
 
-          if prefix && path.index(prefix) == 0
+          if prefix && path.index(prefix) == 0 # rubocop:disable all
             path.sub!(prefix, '')
-            path = Rack::Mount::Utils.normalize_path(path)
+            path = Grape::Router.normalize_path(path)
           end
 
           pieces = path.split('/')
           potential_version = pieces[1]
-          if potential_version =~ options[:pattern]
-            if options[:versions] && !options[:versions].find { |v| v.to_s == potential_version }
-              throw :error, status: 404, message: '404 API Version Not Found'
-            end
-            env['api.version'] = potential_version
-          end
+          return unless potential_version =~ options[:pattern]
+          throw :error, status: 404, message: '404 API Version Not Found' if options[:versions] && !options[:versions].find { |v| v.to_s == potential_version }
+          env[Grape::Env::API_VERSION] = potential_version
         end
 
         private
 
+        def mounted_path?(path)
+          return false unless mount_path && path.start_with?(mount_path)
+          rest = path.slice(mount_path.length..-1)
+          rest.start_with?('/') || rest.empty?
+        end
+
+        def mount_path
+          @mount_path ||= options[:mount_path] && options[:mount_path] != '/' ? options[:mount_path] : ''
+        end
+
         def prefix
-          Rack::Mount::Utils.normalize_path(options[:prefix].to_s) if options[:prefix]
+          Grape::Router.normalize_path(options[:prefix].to_s) if options[:prefix]
         end
       end
     end
