@@ -1,7 +1,7 @@
 module Grape
   module Presenters
     class Presenter
-      def self.represent(object, _options = {})
+      def self.represent(object, **_options)
         object
       end
     end
